package ru.dbotthepony.mc.otm.container

import com.google.common.collect.ImmutableList
import com.mojang.serialization.Codec
import com.mojang.serialization.codecs.RecordCodecBuilder
import net.minecraft.tags.TagKey
import net.minecraft.world.item.Item
import net.minecraft.world.item.ItemStack

data class ItemFilter(val filter: List<ItemStack>, val isWhitelist: Boolean = false, val matchTag: Boolean = false, val matchComponents: Boolean = false) {
	fun add(item: ItemStack): ItemFilter {
		if (filter.any { ItemStack.isSameItemSameComponents(it, item) })
			return this

		return copy(
			filter = ArrayList(filter).also { it.add(item) }
		)
	}

	fun remove(item: ItemStack): ItemFilter {
		val indexOf = filter.indexOfFirst { ItemStack.isSameItemSameComponents(it, item) }

		if (indexOf == -1)
			return this

		return copy(
			filter = ArrayList(filter).also { it.removeAt(indexOf) }
		)
	}

	fun get(index: Int): ItemStack {
		return filter.getOrElse(index) { ItemStack.EMPTY }
	}

	fun isWhitelist(flag: Boolean): ItemFilter {
		if (flag == isWhitelist)
			return this
		else
			return copy(isWhitelist = flag)
	}

	fun matchTag(flag: Boolean): ItemFilter {
		if (flag == matchTag)
			return this
		else
			return copy(matchTag = flag)
	}

	fun matchComponents(flag: Boolean): ItemFilter {
		if (flag == matchComponents)
			return this
		else
			return copy(matchComponents = flag)
	}

	fun match(value: ItemStack): Boolean {
		if (value.isEmpty) {
			return false
		}

		if (filter.isEmpty()) {
			return !isWhitelist
		}

		for (item in filter) {
			var matches = item.`is`(value.item)

			if (matches && matchTag) {
				matches = false

				val thisTags = item.tags
				val stackTags = HashSet<TagKey<Item>>()

				for (tag in value.tags) {
					stackTags.add(tag)
				}

				for (tag1 in thisTags) {
					if (stackTags.contains(tag1)) {
						matches = true
						break
					}
				}
			}

			if (matches && matchComponents) {
				matches = item.components == value.components
			}

			if (matches) {
				return isWhitelist
			}
		}

		return !isWhitelist
	}

	companion object {
		val EMPTY = ItemFilter(ImmutableList.of())

		val CODEC: Codec<ItemFilter> by lazy {
			RecordCodecBuilder.create<ItemFilter> {
				it.group(
					Codec.list(ItemStack.CODEC, 0, 40).fieldOf("filter").forGetter { it.filter },
					Codec.BOOL.optionalFieldOf("isWhitelist", false).forGetter { it.isWhitelist },
					Codec.BOOL.optionalFieldOf("matchTag", false).forGetter { it.matchTag },
					Codec.BOOL.optionalFieldOf("matchComponents", false).forGetter { it.matchComponents },
				).apply(it, ::ItemFilter)
			}
		}
	}
}
