package ru.dbotthepony.mc.otm.network

import it.unimi.dsi.fastutil.io.FastByteArrayOutputStream
import net.minecraft.network.FriendlyByteBuf
import net.minecraft.network.codec.StreamCodec
import net.minecraft.network.protocol.common.custom.CustomPacketPayload
import net.neoforged.neoforge.network.handling.IPayloadContext
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import ru.dbotthepony.mc.otm.capability.matteryPlayer
import ru.dbotthepony.mc.otm.client.minecraft
import ru.dbotthepony.mc.otm.core.ResourceLocation
import ru.dbotthepony.mc.otm.menu.ExopackInventoryMenu
import ru.dbotthepony.mc.otm.menu.MatteryMenu
import java.io.ByteArrayInputStream

class MenuDataPacket(val containerId: Int, val bytes: ByteArray, val length: Int) : CustomPacketPayload {
	constructor(containerId: Int, stream: FastByteArrayOutputStream) : this(containerId, stream.array, stream.length)

	fun write(buff: FriendlyByteBuf) {
		buff.writeVarInt(containerId)
		buff.writeBytes(bytes, 0, length)
	}

	fun play(context: IPayloadContext) {
		if (containerId == ExopackInventoryMenu.CONTAINER_ID) {
			minecraft.player?.matteryPlayer?.exoPackMenu?.mSynchronizer?.read(ByteArrayInputStream(bytes, 0, length))
		} else {
			val menu = minecraft.player?.containerMenu as? MatteryMenu ?: return

			if (menu.containerId == containerId)
				menu.mSynchronizer.read(ByteArrayInputStream(bytes, 0, length))
		}
	}

	override fun type(): CustomPacketPayload.Type<out CustomPacketPayload> {
		return TYPE
	}

	companion object {
		val TYPE = CustomPacketPayload.Type<MenuDataPacket>(
			ResourceLocation(
				OverdriveThatMatters.MOD_ID,
				"menu_data"
			)
		)

		val CODEC: StreamCodec<FriendlyByteBuf, MenuDataPacket> =
			StreamCodec.ofMember(MenuDataPacket::write, ::read)

		fun read(buff: FriendlyByteBuf): MenuDataPacket {
			val containerId = buff.readVarInt()
			val readable = buff.readableBytes()
			return MenuDataPacket(containerId, ByteArray(readable).also(buff::readBytes), readable)
		}
	}
}
